% 
%   MLCV Winter 2015
%   Homework Assignment 03 - Gaussian Mixture Models
%
%   Exercise 2a) K-Means algorithm
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [ clusterIdx ] = mkmeans( X, K, varargin )
%MKMEANS Summary of this function goes here
% X - nxdim matrix, the data: n samples with dim properties each
% K - number of clusters to assign the data to

if nargin > 2
    plot = varargin{1};
end

[~, dim] = size(X);
Xmin = min(X);
Xmax = max(X);

% Initialize cluster means randomly within the data range
M = computeRandomMeans();

if plot
    mscatter(X);
    hold on
    mscatter(M, (1:K)', 'x'); 
end

% Convergence criteria
eTol = 1e-2; 
converged = 0;
maxIter = 100;

iter = 1;
while ~converged && iter < maxIter

    % Assign samples to nearest mean's cluster
    DistToMeans = pdist2(X,M);
    [minDist, clusterIdx] = min(DistToMeans, [], 2); 
    %cost = sum(minDist) % J
    
    % Reinitialize means if a cluster has no samples assigned to it
    while (length(unique(clusterIdx)) ~= K)
        % Initialize means randomly
        M = computeRandomMeans();
        
        % Assign samples to nearest mean's cluster
        DistToMeans = pdist2(X,M);
        [minDist, clusterIdx] = min(DistToMeans, [], 2); 
    end
    
    % Visualize the assignment
    if plot, 
        mscatter(X, clusterIdx);
        mscatter(M, (1:K)', 'x'); 
        legend(['Iter: ' num2str(iter)]);
    end

    % Compute new mean of each cluster
    Mold = M; % save old cluster means
    for ci=1:K
        M(ci,:) = mean(X(clusterIdx == ci, :));
    end

    % Compute the mean change for each cluster
    meanChange = zeros(K, 1);
    for ci=1:K
        meanChange(ci) = norm(M(ci,:) - Mold(ci,:));
    end    
    
    % Evaluate converge criterion
    if all(meanChange < eTol)
        converged = true;
    end
    iter = iter + 1;
end

if converged
    fprintf('Converged after %d iterations\n', iter);
else
    fprintf('Did not converge after %d iterations\n', iter);
end

function [M] = computeRandomMeans()
    %C = X(randi(N, k, 1), :);
    M = zeros(K,dim);
   
    % Compute new random means
    for c=1:K
        M(c,:) = Xmin + (Xmax-Xmin).*rand(1,dim);
    end  
end

end
